#!/bin/sh

########################################################################
# Copyright (C) 2007-2009 Daniel Balsiger <mail<at>danielbalsiger.ch>  #
#                                                                      #
# This project evolved while my information science project and my     #
# master thesis that followed on.                                      #
#                                                                      #
# Check for latest versions at:                                        #
# https://danielbalsiger.ch/svn/repos/trunk/image-builder              #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License version 2 as    #
# published by the Free Software Foundation.                           #
#                                                                      #
########################################################################

. /etc/init.d/rc.functions
. /etc/conf.d/rc.conf

if [ "${GLOBAL_UPDATE_TYPE}" != "save" ] ; then
   echo "Only supported on save update platforms."
   echo "GLOBAL_UPDATE_TYPE is not set to 'save'."
   exit 1
fi

UPDATE_DIR=/var/lib/update
ACTUAL_VERSION=`cat /var/lib/image-version`
if [ -f /mnt/state/log/old-version ] ; then
   OLD_VERSION=`cat /mnt/state/log/old-version`
else
   OLD_VERSION="none"
fi
if grep -e "FALLBACK" /proc/cmdline 2>/dev/null ; then
   UPDATE_MODE="FALLBACK"
elif grep -e "UPDATE" /proc/cmdline 2>/dev/null ; then
   UPDATE_MODE="UPDATE"
else
   UPDATE_MODE="DEFAULT"
fi

usage()
{
    echo "Usage: ${0} init [new_version] | switch | fallback"
}

case $1 in
    init)
        if [ -z ${2} ] ; then
            usage
            exit 1
        fi
        if [ "${UPDATE_MODE}" != "DEFAULT" ] ; then
            logecho "Init is possible only in default mode"
            exit 1
        fi
        if [ ! -f ${UPDATE_DIR}/${GLOBAL_SYSTYPE}-image-${2}.bin.gz ] ; then
            logecho "No image ${UPDATE_DIR}/${GLOBAL_SYSTYPE}-image-${2}.bin.gz found"
            exit 1
        fi
        if [ "${2}" == "${ACTUAL_VERSION}" ] ; then
            logecho "$0 WARNING: init with ${ACTUAL_VERSION}..."
            exit 1
        fi
        mount /boot
        if ! grep UPDATE_VERSION /boot/grub/menu.lst 2>/dev/null ; then
            logecho "Seems update was already initalized..."
            umount /boot
            exit 1
        fi
        install -o root -g root -m 0644 ${UPDATE_DIR}/${GLOBAL_SYSTYPE}-image-${2}.bin.gz /boot
        sed 's/UPDATE_VERSION/'`echo ${2}`'/' -i /boot/grub/menu.lst
        sed 's/default 0/default 1/' -i /boot/grub/menu.lst
        sed 's|rdinit=/sbin/init loglevel=0|rdinit=/sbin/init loglevel=0 FALLBACK|' -i /mnt/state/grub/menu.lst
        umount /boot
        logecho "Update initalized, reboot now to test it."
        reboot
        ;;
    switch)
        if [ "${UPDATE_MODE}" != "UPDATE" ] ; then
            logecho "Switch is only possible in update mode."
            exit 1
        fi
        if [ "${OLD_VERSION}" == "${ACTUAL_VERSION}" ] ; then
            logecho "$0 WARNING: switch to ${ACTUAL_VERSION}..."
            exit 1
        fi
        mount /boot
        if grep UPDATE_VERSION /boot/grub/menu.lst 2>/dev/null ; then
            logecho "It seems switch was already done."
            umount /boot
            exit 1
        fi
        newimage="${GLOBAL_SYSTYPE}-image-${ACTUAL_VERSION}.bin.gz"
        emptyimage="${GLOBAL_SYSTYPE}-image-UPDATE_VERSION.bin.gz"
        oldimage="${GLOBAL_SYSTYPE}-image-${OLD_VERSION}.bin.gz"
        sed 's/default 1/default 0/' -i /boot/grub/menu.lst
        sed 's/'`echo ${newimage}`'/'`echo ${emptyimage}`'/' -i /boot/grub/menu.lst
        sed 's/'`echo ${oldimage}`'/'`echo ${newimage}`'/' -i /boot/grub/menu.lst
        sed 's/'`echo ${oldimage}`'/'`echo ${newimage}`'/' -i /mnt/state/grub/menu.lst
        sed 's|rdinit=/sbin/init loglevel=0 FALLBACK|rdinit=/sbin/init loglevel=0|' -i /mnt/state/grub/menu.lst
        umount /boot
        logecho "Update installed as default, reboot now to enable future updates."
        ;;
    fallback)
        if [ "${UPDATE_MODE}" != "FALLBACK" ] ; then
            logecho "Fallback is only possible in fallback mode."
            exit 1
        fi
        mount /boot
        if grep UPDATE_VERSION /boot/grub/menu.lst 2>/dev/null ; then
            logecho "It seems fallback was already done."
            umount /boot
            exit 1
        fi
        newimage="${GLOBAL_SYSTYPE}-image-${ACTUAL_VERSION}.bin.gz"
        emptyimage="${GLOBAL_SYSTYPE}-image-UPDATE_VERSION.bin.gz"
        standalone="${GLOBAL_SYSTYPE}-image-standalone-${ACTUAL_VERSION}.bin.gz"
        cat >/boot/grub/menu.lst << EOF
# /boot/grub/menu.lst
# default menu.lst on hda1

#password --md5 [Generate one with 'grub-md5-crypt']
hiddenmenu
default 0
fallback 0
timeout 5
serial --unit=0 --speed=9600 --word=8 --parity=no --stop=1
terminal serial

title default
root(hd0,0)
EOF
        echo "kernel /${newimage} console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0" >> /boot/grub/menu.lst
        echo "" >> /boot/grub/menu.lst
        echo "title update" >> /boot/grub/menu.lst
        echo "root (hd0,1)" >> /boot/grub/menu.lst
        echo "setup (hd0)" >> /boot/grub/menu.lst
        echo "root (hd0,0)" >> /boot/grub/menu.lst
        echo "kernel /${emptyimage} console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0 UPDATE" >> /boot/grub/menu.lst
        echo "" >> /boot/grub/menu.lst
        echo "title standalone" >> /boot/grub/menu.lst
        echo "root(hd0,0)" >> /boot/grub/menu.lst
        echo "kernel /${standalone} console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0" >> /boot/grub/menu.lst
        sed 's|rdinit=/sbin/init loglevel=0 FALLBACK|rdinit=/sbin/init loglevel=0|' -i /mnt/state/grub/menu.lst
        umount /boot
        logecho "Fallback to the now running image, reboot to enable normal operation."
        ;;
    *)
        usage
        ;;
esac

