#!/bin/sh

########################################################################
# Copyright (C) 2007-2009 Daniel Balsiger <mail<at>danielbalsiger.ch>  #
#                                                                      #
# This project evolved while my information science project and my     #
# master thesis that followed on.                                      #
#                                                                      #
# Check for latest versions at:                                        #
# https://danielbalsiger.ch/svn/repos/trunk/image-builder              #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License version 2 as    #
# published by the Free Software Foundation.                           #
#                                                                      #
########################################################################

# calculate values

. /etc/conf.d/network.conf

if [ -f /tmp/cfpeers ] ; then
    gen_peers="`cat /tmp/cfpeers`"
else
    gen_peers=""
fi

if [ -f /tmp/nopeers ] ; then
    now=`date -u '+%s'`
    lostsince=`date -u '+%s' -r /tmp/nopeers`
    let elapsed=now-lostsince
    let elapsed=elapsed/60
    gen_peers="No peers since ${elapsed} minutes. (Timeout is ${GLOBAL_LOSTTIMEOUT} hours)"
fi

if [ -f /mnt/state/log/old-version ] ; then
    old_image="`cat /mnt/state/log/old-version`"
else
    old_image="n/a"
fi

filter4_status="disabled"
filter6_status="disabled"
ntpd_status="disabled"
radvd_status="disabled"
udhcpd_status="disabled"

if [ ${NETFILTER6_START} == "yes" ] ; then
    filter6_status="enabled"
fi

if [ ${NETFILTER_START} == "yes" ] ; then
    filter4_status="enabled"
fi

if [ ${GLOBAL_NTP_BE_SERVER} == "yes" ] ; then
    ntpd_status="enabled"
fi

if [ ${GLOBAL_RADVD_START} == "yes" ] ; then
    radvd_status="enabled"
fi

if [ ${GLOBAL_UDHCPD_START} == "yes" ] ; then
    udhcpd_status="enabled"
fi

gen_image="`cat /var/lib/image-version`"

gen_sysname="`cat /etc/conf.d/hostname`.`cat /etc/conf.d/domainname`"
gen_kernel=`uname -sr`
gen_date=`date`
gen_load=`uptime`
gen_load=${gen_load#*average:}
gen_uptime=`uptime`
gen_uptime=${gen_uptime#*up}
gen_uptime=${gen_uptime%, *user*}
gen_uptime=${gen_uptime%*, load*}

mem_used=`free | grep Mem |  awk  '{print $3}'`
mem_used="$mem_used kB"
mem_total=`free | grep Mem |  awk  '{print $2}'`
mem_total="$mem_total kB"
mem_cached=`cat /proc/meminfo | grep Cached | head -1 | awk '{print $2}'`
mem_cached="$mem_cached kB"
mem_buffer=`cat /proc/meminfo | grep Buffer | awk '{print $2}'`
mem_buffer="$mem_buffer kB"
mem_free=`free | grep Mem |  awk  '{print $4}'`
mem_free="$mem_free kB"

cpu_vendor=`cat /proc/cpuinfo | grep type | awk  '{print $4}'`
if [ -z "$cpu_vendor" ] ; then
cpu_vendor=`cat /proc/cpuinfo | grep vendor | awk  '{print $5}'`
fi
if [ -z "$cpu_vendor" ] ; then
cpu_vendor=`cat /proc/cpuinfo | grep vendor | awk  '{print $3}'`
fi
cpu_type=`cat /proc/cpuinfo | grep type | awk '{print $5}'`
if [ -z "$cpu_type" ] ; then
cpu_type=`cat /proc/cpuinfo | grep MHz | awk '{print $4}'`
fi
cpu_model=`cat /proc/cpuinfo | grep model | awk '{print $4} {print $5} {print $6}'`
if echo $cpu_model | grep Unknown >/dev/null ; then
cpu_model=`cat /proc/cpuinfo | grep vendor | awk '{print $3}'`
fi
cpu_arch=`uname -m`
cpu_bogo=`cat /proc/cpuinfo | grep ogo | awk  '{print $3}'`


# begin output
echo "Content-type: text/html; charset=iso-8859-1"
echo ""

# header
echo "<!DOCTYPE html PUBLIC '-//W3C//DTD XHTML 1.0 Strict//EN' 'http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd'>"
echo "<html xmlns='http://www.w3.org/1999/xhtml' xml:lang='en' lang='en'>"
echo "<head>"
echo "<meta http-equiv='content-type' content='text/html;charset=iso-8859-1'></meta>"
echo "<link rel='shortcut icon' href='/pics/favicon.ico'></link>"
echo "<style type='text/css' media='screen'> @import url('/pics/style.css');</style>"
echo "<title>${gen_sysname}</title>"
echo "</head>"
echo "<body>"
echo "<div class='pics'>"
echo "<table width='100%'><tr>"
echo "<td valign='top' align='center'><img src='/pics/nodebig_left.png' alt='nodebig_left' /></td>"
echo "<td valign='top' align='center'><img src='/pics/favicon.ico' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/node.png' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/favicon.ico' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/node.png' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/favicon.ico' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/node.png' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/favicon.ico' alt='node' /></td>"
echo "<td valign='top' align='center'><img src='/pics/nodebig_right.png' alt='nodebig_right' /></td>"
echo "</tr></table>"
echo "<div class='title'>"
echo "Administration and Deployment of Wireless Mesh Networks"
echo "</div>"
echo "</div>"
echo "<div class='navbar'>"
echo "<div class='navlinks'>"
echo "<table width='100%'><tr>"
echo "<td valign='top' align='left'>"
echo "<div class='content'>"

# content
echo "<div class='big'><img src='/pics/favicon.ico' alt='node' />System Information of $gen_sysname</div>"

echo "<table cellpadding='0' cellspacing='0' >"
echo "<tr>"
echo "<td class='border' colspan='6' align='left' ><strong>Overview</strong></td>"
echo "</tr>"
echo "<td class='border' colspan='2' align='left' ><strong>General Settings</strong></td>"
echo "<td class='border' colspan='2' align='left' ><strong>CPU Information</strong></td>"
echo "<td class='border' colspan='2' align='left' ><strong>Memory Information</strong></td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Node Type</td>"
echo "<td class='border' align='right'>${GLOBAL_SYSTYPE} [ Mgmt Node: ${GLOBAL_MGMT} ]</td>"
echo "<td class='noside' align='left'>Vendor</td>"
echo "<td class='border' align='right'>${cpu_vendor}</td>"
echo "<td class='noside' align='left'>Total</td>"
echo "<td class='border' align='right'>${mem_total}</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Image Version</td>"
echo "<td class='border' align='right'>${gen_image} [ Old version: ${old_image} ]</td>"
echo "<td class='noside' align='left'>Type/MHz</td>"
echo "<td class='border' align='right'>${cpu_type}</td>"
echo "<td class='noside' align='left'>Used</td>"
echo "<td class='border' align='right'>${mem_used}</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Uptime / Load</td>"
echo "<td class='border' align='right'>${gen_uptime} / ${gen_load}</td>"
echo "<td class='noside' align='left'>Model</td>"
echo "<td class='border' align='right'>${cpu_model}</td>"
echo "<td class='noside' align='left'>Cached</td>"
echo "<td class='border' align='right'>${mem_cached}</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Kernel</td>"
echo "<td class='border' align='right'>${gen_kernel}</td>"
echo "<td class='noside' align='left'>Architecture</td>"
echo "<td class='border' align='right'>${cpu_arch}</td>"
echo "<td class='noside' align='left'>Buffers</td>"
echo "<td class='border' align='right'>${mem_buffer}</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Date</td>"
echo "<td class='border' align='right'>${gen_date}</td>"
echo "<td class='noside' align='left'>BogoMIPS</td>"
echo "<td class='border' align='right'>${cpu_bogo}</td>"
echo "<td class='noside' align='left'>Free</td>"
echo "<td class='border' align='right'>${mem_free}</td>"
echo "</tr>"
echo "</table>"

echo "<div class='big'></div>"

echo "<table cellpadding='0' cellspacing='0' >"
echo "<tr>"
echo "<td class='border' colspan='6' align='left' ><strong>Network Settings</strong></td>"
echo "</tr>"
echo "<tr>"
echo "<td class='border' colspan='2' align='left' ><strong>Cfengine Information</strong></td>"
echo "<td class='border' colspan='2' align='left' ><strong>External Resources</strong></td>"
echo "<td class='border' colspan='2' align='left' ><strong>Local Resources</strong></td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left' >Peers Found</td>"
echo "<td class='border' colspan='1' align='right' >${gen_peers}</td>"
echo "<td class='noside' align='left' >IPv4 Default Route</td>"
echo "<td class='border' colspan='1' align='right' >`ip -4 route | grep default | sed 's/default//'`</td>"
echo "<td class='noside' align='left' >IPv4 DHCP Server</td>"
echo "<td class='border' colspan='1' align='right' >$udhcpd_status</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left' >Search Over</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_CFHOPS} hops</td>"
echo "<td class='noside' align='left' >IPv6 Default Route</td>"
echo "<td class='border' colspan='1' align='right' >`ip -6 route | grep default | sed 's/default//'`</td>"
echo "<td class='noside' align='left' >IPv4/IPv6 NTP Server</td>"
echo "<td class='border' colspan='1' align='right' >$ntpd_status</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left' >Cfengine Prefix</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_CFNET_PREFIX}</td>"
echo "<td class='noside' align='left' >DNS Servers</td>"
echo "<td class='border' colspan='1' align='right' >`cat /etc/resolv.conf | grep nameserver | sed -e 's/nameserver//'`</td>"
echo "<td class='noside' align='left' >IPv6 RADV Server</td>"
echo "<td class='border' colspan='1' align='right' >$radvd_status</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Cfengine MAC</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_CFNET_MAC}</td>"
echo "<td class='noside' align='left'>NTP Servers</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_NTP_SERVER}</td>"
echo "<td class='noside' align='left'>IPv4 Netfilter</td>"
echo "<td class='border' colspan='1' align='right' >$filter4_status</td>"
echo "</tr>"
echo "<tr>"
echo "<td class='noside' align='left'>Configured Interfaces</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_DEVICES}</td>"
echo "<td class='noside' align='left'>Syslog Server</td>"
echo "<td class='border' colspan='1' align='right' >${GLOBAL_SYSLOG_SERVER}</td>"
echo "<td class='noside' align='left'>IPv6 Netfilter</td>"
echo "<td class='border' colspan='1' align='right' >$filter6_status</td>"
echo "</tr>"
echo "</table>"

echo "<div class='big'></div>"

echo "<table cellpadding='0' cellspacing='0' >"
echo "<tr>"
echo "<td class='border' colspan='6' align='left' ><strong>Interface States</strong></td>"
echo "</tr>"
echo "<tr>"
echo "<td class='border'><strong>Interface</strong></td>"
echo "<td class='border'><strong>Link State</strong></td>"
echo "<td class='border'><strong>IPv4 Addresses</strong></td>"
echo "<td class='border'><strong>IPv6 Addresses</strong></td>"
echo "<td class='border'><strong>Received Bytes/Packets</strong></td>"
echo "<td class='border'><strong>Transmitted Bytes/Packets</strong></td>"
echo "</tr>"

for dev in ${AVAIL_DEVICES} lo ; do
echo "<tr>"
echo "<td align='left' class='border'><strong>${dev}</strong></td>"
if ip link | grep UP | grep ${dev} > /dev/null ; then
    echo "<td align='right' class='border'><strong>Up</strong> `ip addr show dev ${dev} | grep ether | awk '{print $2}'`</td>"
    echo "<td align='right' class='border'>`ip addr show dev ${dev} | grep "inet " | awk '{print $2}'`</td>"
    echo "<td align='right' class='border'>`ip addr show dev ${dev} | grep inet6 | awk '{print $2}'`</td>"
else
    echo "<td align='right' class='border'><strong>Down</strong></td>"
    echo "<td align='right' class='border'> </td>"
    echo "<td align='right' class='border'> </td>"
fi
echo "<td align='right' class='border'>`cat /proc/net/dev | grep " ${dev}" | sed -e 's|'${dev}:'||' | awk '{print $1}'`/`cat /proc/net/dev | grep " ${dev}" | sed -e 's|'${dev}:'||' | awk '{print $2}'`</td>"
echo "<td align='right' class='border'>`cat /proc/net/dev | grep " ${dev}" | sed -e 's|'${dev}:'||' | awk '{print $9}'`/`cat /proc/net/dev | grep " ${dev}" | sed -e 's|'${dev}:'||' | awk '{print $10}'`</td>"
echo "</tr>"
done
echo "</table>"

echo "<div class='big'></div>"

# footer
echo "</div>"
echo "</td></tr></table>"
echo "</div>"
echo "</div>"
echo "</body>"
echo "</html>"

